"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.sql2aql = exports.mapProjectStatement = exports.mapLimitStatement = exports.mapOrderByStatement = exports.mapAggrStatement = exports.mapGroupByStatement = exports.mapWhereStatement = exports.mapFromStatment = exports.indent = void 0;
const pgsql_ast_parser_1 = require("pgsql-ast-parser");
const functionMap = {
    count: 'COUNT',
    min: 'MIN',
    max: 'MAX',
    sum: 'SUM',
    avg: 'AVG'
};
const indentMap = {};
function indent(level, size = 2) {
    if (!indentMap[level]) {
        indentMap[level] = ' '.repeat(level * size);
    }
    return indentMap[level];
}
exports.indent = indent;
function mapFromStatment(fromAst, ctx) {
    if (fromAst.length !== 1) {
        throw new Error(`Invalid from ast! ${fromAst.length} statement(s)`);
    }
    return `FOR ${ctx.docRef} IN ${fromAst[0]['name']['name']}`;
}
exports.mapFromStatment = mapFromStatment;
function mapOpStat(expr, deep = 0) {
    switch (expr.type) {
        case 'ref': {
        }
        case 'boolean':
        case 'integer':
        case 'string':
            return expr['value'];
        case 'binary': {
            const op = expr['op'];
            const lhs = mapOpStat(expr['left']);
            const rhs = mapOpStat(expr['right']);
            return deep > 0 ? `(${lhs} ${op} ${rhs})` : `${lhs} ${op} ${rhs}`;
        }
        default:
            throw Error(`Unsupported where expr type ${expr.type}!`);
    }
}
function mapWhereStatement(whereAst) {
    let filterStr = mapOpStat(whereAst);
    if (filterStr) {
        return `FILTER ${filterStr}`;
    }
    return undefined;
}
exports.mapWhereStatement = mapWhereStatement;
function mapGroupByStatement(groupByAsts, columns, ctx) {
    const collectArr = [];
    ctx.collectMap = {};
    for (const groupByAst of groupByAsts) {
        switch (groupByAst.type) {
            case 'integer':
                const groupCol = columns[groupByAst.value - 1];
                const collectEl = `${groupCol.alias.name} = ${ctx.docRef}.${groupCol.expr['name']}`;
                ctx.collectMap[groupCol.alias.name] = collectEl;
                collectArr.push(collectEl);
                break;
            default:
                throw Error(`Unsupported groupBy expr type ${groupByAst.type}!`);
        }
    }
    return `COLLECT ${collectArr.join(',')}`;
}
exports.mapGroupByStatement = mapGroupByStatement;
function mapAggrStatement(columns, ctx) {
    let aggArr = [];
    for (const col of columns) {
        if (col.expr.type === 'call') {
            let aqlFunc = functionMap[col.expr.function.name];
            if (aqlFunc) {
                let aggrEl = `${col.alias.name} = ${aqlFunc}(${col.expr.args.map((expr) => `${ctx.docRef}.${expr['name']}`).join(',')})`;
                ctx.collectMap[col.alias.name] = aggrEl;
                aggArr.push(aggrEl);
            }
        }
    }
    if (aggArr.length) {
        return `AGGREGATE ${aggArr.join(',')}`;
    }
    return undefined;
}
exports.mapAggrStatement = mapAggrStatement;
function mapOrderByStatement(orderByAsts, columns, ctx) {
    const orderByArr = [];
    for (const orderByAst of orderByAsts) {
        switch (orderByAst.by.type) {
            case 'integer':
                const orderCol = columns[orderByAst.by.value - 1];
                const orderByEl = `${orderCol.alias.name} ${orderByAst.order}`;
                // ctx.collectMap[groupCol.alias.name] = collectEl;
                orderByArr.push(orderByEl);
                break;
            default:
                throw Error(`Unsupported orderBy by type ${orderByAst.by.type}!`);
        }
    }
    return `SORT ${orderByArr.join(',')}`;
}
exports.mapOrderByStatement = mapOrderByStatement;
function mapLimitStatement(limitAst) {
    let limitStr = '';
    if (limitAst.limit) {
        switch (limitAst.limit.type) {
            case 'integer':
                limitStr = `${limitAst.limit.value}`;
                break;
            default:
                throw Error(`Unsupported limit type ${limitAst.limit.type}!`);
        }
    }
    return `LIMIT ${limitStr}`;
}
exports.mapLimitStatement = mapLimitStatement;
function mapProjectStatement(columns, ctx) {
    if (columns.length === 1 && columns[0].expr.type === 'ref' && columns[0].expr.name === '*') {
        return `RETURN ${ctx.docRef}`;
    }
    const returnArr = [];
    for (const col of columns) {
        if (ctx.collectMap && ctx.collectMap[col.alias.name]) {
            returnArr.push(`${col.alias.name}`);
        }
        else {
            switch (col.expr.type) {
                case 'ref':
                    returnArr.push(`${col.alias.name}:${ctx.docRef}.${col.expr.name}`);
                    break;
                default:
                    throw Error(`Unsupported projection expr type ${col.expr.type}!`);
            }
        }
    }
    return `RETURN {${returnArr.join(',')}}`;
}
exports.mapProjectStatement = mapProjectStatement;
function sql2aql(sql) {
    const ast = (0, pgsql_ast_parser_1.parse)(sql);
    let aqlQuery = '';
    let firstAst = ast[0];
    if (firstAst.type === 'select') {
        let ctx = { docRef: 'doc' };
        aqlQuery = `${mapFromStatment(firstAst.from, ctx)}\n`;
        if (firstAst.where) {
            let filterStr = mapWhereStatement(firstAst.where);
            if (filterStr) {
                aqlQuery += `${indent(1)}${filterStr}\n`;
            }
        }
        if (firstAst.groupBy) {
            aqlQuery += `${indent(1)}${mapGroupByStatement(firstAst.groupBy, firstAst.columns, ctx)}\n`;
            let aggrStr = mapAggrStatement(firstAst.columns, ctx);
            if (aggrStr) {
                aqlQuery += `${indent(1)}${aggrStr}\n`;
            }
        }
        if (firstAst.orderBy) {
            aqlQuery += `${indent(1)}${mapOrderByStatement(firstAst.orderBy, firstAst.columns, ctx)}\n`;
        }
        if (firstAst.limit) {
            aqlQuery += `${indent(1)}${mapLimitStatement(firstAst.limit)}\n`;
        }
        aqlQuery += `${indent(1)}${mapProjectStatement(firstAst.columns, ctx)}`;
    }
    return aqlQuery;
}
exports.sql2aql = sql2aql;
//# sourceMappingURL=sql-utils.js.map